@@ -55,8 +55,7 @@ struct App
   GtkWidget      *panel;
   GtkWidget      *current_monitor_event_box;
   GtkWidget      *current_monitor_label;
-  GtkWidget      *monitor_on_radio;
-  GtkWidget      *monitor_off_radio;
+  GtkWidget      *monitor_switch;
   GtkListStore   *resolution_store;
   GtkWidget      *resolution_combo;
   GtkWidget      *rotation_combo;
@@ -87,7 +86,7 @@ static void rebuild_gui (App *app);
 static void on_clone_changed (GtkWidget *box, gpointer data);
 static gboolean output_overlaps (GnomeOutputInfo *output, GnomeRRConfig *config);
 static void select_current_output_from_dialog_position (App *app);
-static void monitor_on_off_toggled_cb (GtkToggleButton *toggle, gpointer data);
+static void monitor_switch_active_cb (GObject *object, GParamSpec *pspec, gpointer data);
 static void get_geometry (GnomeOutputInfo *output, int *w, int *h);
 static void apply_configuration_returned_cb (DBusGProxy *proxy, DBusGProxyCall *call_id, void *data);
 static gboolean get_clone_size (GnomeRRScreen *screen, int *width, int *height);
@@ -383,7 +382,7 @@ count_all_outputs (GnomeRRConfig *config)
 }
 #endif
 
-/* Computes whether "Mirror Screens" (clone mode) is supported based on these criteria:
+/* Computes whether "Mirror displays" (clone mode) is supported based on these criteria:
  *
  * 1. There is an available size for cloning.
  *
@@ -445,6 +444,17 @@ rebuild_mirror_screens (App *app)
   g_signal_handlers_unblock_by_func (app->clone_checkbox, G_CALLBACK (on_clone_changed), app);
 }
 
+static char *
+mirror_monitor_name (void)
+{
+  /* Translators:  this is the feature where what you see on your laptop's
+   * screen is the same as your external monitor.  Here, "Mirror" is being
+   * used as an adjective, not as a verb.  For example, the Spanish
+   * translation could be "Pantallas en Espejo", *not* "Espejar Pantallas".
+   */
+  return g_strdup (_("Mirror Displays"));
+}
+
 static void
 rebuild_current_monitor_label (App *app)
 {
@@ -455,7 +465,7 @@ rebuild_current_monitor_label (App *app)
   if (app->current_output)
     {
       if (app->current_configuration->clone)
-        tmp = g_strdup (_("Mirror Screens"));
+        tmp = mirror_monitor_name ();
       else
         tmp = g_strdup (app->current_output->display_name);
 
@@ -504,8 +514,7 @@ rebuild_on_off_radios (App *app)
   gboolean on_active;
   gboolean off_active;
 
-  g_signal_handlers_block_by_func (app->monitor_on_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
-  g_signal_handlers_block_by_func (app->monitor_off_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
+  g_signal_handlers_block_by_func (app->monitor_switch, G_CALLBACK (monitor_switch_active_cb), app);
 
   sensitive = FALSE;
   on_active = FALSE;
@@ -522,14 +531,11 @@ rebuild_on_off_radios (App *app)
       off_active = !on_active;
     }
 
-  gtk_widget_set_sensitive (app->monitor_on_radio, sensitive);
-  gtk_widget_set_sensitive (app->monitor_off_radio, sensitive);
+  gtk_widget_set_sensitive (app->monitor_switch, sensitive);
 
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->monitor_on_radio), on_active);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->monitor_off_radio), off_active);
+  gtk_switch_set_active (GTK_SWITCH (app->monitor_switch), on_active);
 
-  g_signal_handlers_unblock_by_func (app->monitor_on_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
-  g_signal_handlers_unblock_by_func (app->monitor_off_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
+  g_signal_handlers_unblock_by_func (app->monitor_switch, G_CALLBACK (monitor_switch_active_cb), app);
 }
 
 static char *
@@ -749,31 +755,28 @@ select_resolution_for_current_output (App *app)
 }
 
 static void
-monitor_on_off_toggled_cb (GtkToggleButton *toggle, gpointer data)
+monitor_switch_active_cb (GObject    *object,
+                          GParamSpec *pspec,
+                          gpointer    data)
 {
   App *app = data;
+  gboolean value;
 
   if (!app->current_output)
     return;
 
-  if (!gtk_toggle_button_get_active (toggle))
-    return;
+  value = gtk_switch_get_active (GTK_SWITCH (object));
 
-  if (GTK_WIDGET (toggle) == app->monitor_on_radio)
+  if (value)
     {
       app->current_output->on = TRUE;
       select_resolution_for_current_output (app);
     }
-  else if (GTK_WIDGET (toggle) == app->monitor_off_radio)
+  else
     {
       app->current_output->on = FALSE;
       gnome_rr_config_ensure_primary (app->current_configuration);
     }
-  else
-    {
-      g_assert_not_reached ();
-      return;
-    }
 
   rebuild_gui (app);
   foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
@@ -987,7 +990,7 @@ on_clone_changed (GtkWidget *box, gpointer data)
         }
 
       /* Turn on all the connected screens that support the best clone mode.
-       * The user may hit "Mirror Screens", but he shouldn't have to turn on
+       * The user may hit "Mirror displays", but he shouldn't have to turn on
        * all the required outputs as well.
        */
 
@@ -1710,16 +1713,10 @@ get_display_name (App *app,
   PangoLayout *layout;
   char *text;
 
-  if (app->current_configuration->clone) {
-    /* Translators:  this is the feature where what you see on your laptop's
-     * screen is the same as your external monitor.  Here, "Mirror" is being
-     * used as an adjective, not as a verb.  For example, the Spanish
-     * translation could be "Pantallas en Espejo", *not* "Espejar Pantallas".
-     */
-    text = g_strdup(_("Mirror Screens"));
-  } else {
+  if (app->current_configuration->clone)
+    text = mirror_monitor_name ();
+  else
     text = g_strdup (output->display_name);
-  }
 
   layout = gtk_widget_create_pango_layout (GTK_WIDGET (app->area), text);
   g_free (text);
@@ -2362,7 +2359,7 @@ get_output_for_window (GnomeRRConfig *configuration, GdkWindow *window)
   int largest_area;
   int largest_index;
 
-  gdk_window_get_geometry (window, &win_rect.x, &win_rect.y, &win_rect.width, &win_rect.height, NULL);
+  gdk_window_get_geometry (window, &win_rect.x, &win_rect.y, &win_rect.width, &win_rect.height);
   gdk_window_get_origin (window, &win_rect.x, &win_rect.y);
 
   largest_area = 0;
@@ -2522,14 +2519,10 @@ run_application (void)
   app->current_monitor_label = _gtk_builder_get_widget (builder,
                                                         "current_monitor_label");
 
-  app->monitor_on_radio = _gtk_builder_get_widget (builder,
-                                                   "monitor_on_radio");
-  app->monitor_off_radio = _gtk_builder_get_widget (builder,
-                                                    "monitor_off_radio");
-  g_signal_connect (app->monitor_on_radio, "toggled",
-                    G_CALLBACK (monitor_on_off_toggled_cb), app);
-  g_signal_connect (app->monitor_off_radio, "toggled",
-                    G_CALLBACK (monitor_on_off_toggled_cb), app);
+  app->monitor_switch = _gtk_builder_get_widget (builder,
+                                                 "monitor_switch");
+  g_signal_connect (app->monitor_switch, "notify::active",
+                    G_CALLBACK (monitor_switch_active_cb), app);
 
   app->resolution_combo = _gtk_builder_get_widget (builder,
                                                    "resolution_combo");
@@ -2560,7 +2553,7 @@ run_application (void)
   set_monitors_tooltip (app, FALSE);
 
   /* FIXME: this should be computed dynamically */
-  foo_scroll_area_set_min_size (FOO_SCROLL_AREA (app->area), -1, 200);
+  foo_scroll_area_set_min_size (FOO_SCROLL_AREA (app->area), 0, 200);
   gtk_widget_show (app->area);
   g_signal_connect (app->area, "paint",
                     G_CALLBACK (on_area_paint), app);
